# ppe-detection-ml-pipeline infrastructure

This is an AWS Serverless Application Model (AWS SAM) project and requires you to have the AWS Command Line Interface (AWS CLI) and AWS SAM CLI set up. 

## Deploying

To build and deploy the AWS CloudFormation stack, complete the following steps:

1.	Configure your AWS CLI credentials.
2.	Install the AWS SAM CLI using these [instructions](https://docs.aws.amazon.com/serverless-application-model/latest/developerguide/serverless-sam-cli-install.html).
3.	Download the [Ppe-detection-ml-pipeline.zip](https://aws-ml-blog.s3.amazonaws.com/artifacts/Build-an-end-to-end-PPE-monitoring-solution-Rekognition-API+/Ppe-detection-ml-pipeline.zip) file.
4.	Unzip the contents of the .zip file and navigate to the root of the project.
5.	Build the project – This will package the Lambda functions.

      `sam build`

6.	Deploy the CloudFormation stack in your AWS account

      `sam deploy –-guided`

      Choose an appropriate AWS region to deploy the stack. Use the defaults for all other prompts.
      Note: use `sam deploy` for subsequent deployments.

## Overview

We use Amazon S3 as a data lake to store the images coming out of the video ingestion pipeline after it splits the original camera feeds into 1 FPS images. The S3 data lake bucket folder structure, which organizes the collected image and camera metadata along with the model responses. Make sure you have the similar folder structure in the deployed S3 bucket for the code to work correctly. 

```
S3 DataLake Bucket
-- input/
---- [camera_id]/ 
-- output/
---- ppe/
------ [YYYY]/[MM]/[DD]/
-- error/
---- ppe/
```

After deploying the stack, create the input folder inside the S3 bucket. The input prefix can contain multiple folders, which helps in organizing the results by camera source. To test the pipeline, upload a jpg containing people and faces to under the input/[camera_id]/ folder in the S3 bucket. The camera_id can be any arbitrary name. The Output and Error prefixes are created automatically when the PPE detection job is triggered. The output prefix contains model inference outputs from multiple images combined together. The error prefix contains records of jobs that failed to run.

For example, this [sample image](https://dhei5unw3vrsx.cloudfront.net/images/ppe_group_updated.jpg) is uploaded to the S3 Bucket under input/CAMERA01/. After a short while, the output/ prefix will contain a file with a JSON record indicating that all 4 individuals were PPE compliant:

```
{
  "CameraID": "CAMERA01",
  "Bucket": "ppe-detection-blog-post-datalakes3bucket-abc123abc123",
  "Key": "input/CAMERA01/ppe_sample_image.jpg",
  "RekResult": "{\"0\": \"compliant\", \"1\": \"compliant\", \"2\": \"compliant\", \"3\": \"compliant\"}",
  "FaceCoverNonCompliant": 0,
  "FaceCoverNonCompliantPercentage": 0,
  "detectedFaceRate": 100
}
```

The provided AWS SAM project creates the resources, roles and necessary configuration of the pipeline. Note that the IAM roles deployed are very permissive and should be locked down for real world use cases.
